<?php
/* Copyright (c) 1998-2013 ILIAS open source, Extended GPL, see docs/LICENSE */

require_once 'Services/Repository/classes/class.ilObjectPlugin.php';
require_once 'models/interface.ilPluginModelInterface.php';
require_once 'Modules/TestQuestionPool/interfaces/interface.ilQuestionEditingFormConsumer.php';

/**
 * Class ilObjQuestionSetPool
 *
 * Date: 23.09.13
 * Time: 13:12
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilObjQuestionSetPool extends ilObjectPlugin implements ilPluginModelInterface, ilQuestionEditingFormConsumer
{
	/**
	 * @var ilQuestionSetPoolPlugin
	 */
	public $plugin;

	/**
	 * Boolean to check weather the object is online
	 *
	 * @var boolean
	 */
	protected $online;

	/**
	 * @var ilQuestionSetPoolSetList
	 */
	protected $set_list;

	/**
	 * @var array
	 */
	protected static $online_status_map = array();

	/**
	 * @var bool
	 */
	private $skillServiceEnabled = false;

	/**
	 * @var null|bool
	 */
	private static $isSkillManagementGloballyActivated = null;

	/**
	 * @param int $a_ref_id
	 */
	public function __construct($a_ref_id = 0)
	{
		global $DIC;

		require_once 'models/class.ilQuestionSetPoolSetList.php';
		parent::__construct($a_ref_id);

		$DIC->language()->loadLanguageModule('assessment');
	}

    public function checkQuestionParent($questionId)
    {
        global $DIC; /* @var ILIAS\DI\Container $DIC */

        $row = $DIC->database()->fetchAssoc($DIC->database()->queryF(
            "SELECT COUNT(question_id) cnt FROM qpl_questions WHERE question_id = %s AND obj_fi = %s",
            array('integer', 'integer'),
            array($questionId, $this->getId())
        ));

        return (bool) $row['cnt'];
    }

	/**
	 * @return string
	 */
	public function getQuestionEditingFormBackTargetLabel()
	{
		return ilQuestionSetPoolPlugin::getInstance()->txt('back_to_qsp');
	}

	/**
	 * @param $context
	 * @return string
	 */
	public function getQuestionEditingFormBackTarget($context)
	{
		global $DIC; 
		$ilCtrl = $DIC->ctrl();
		$params = @json_decode(base64_decode($context), true);

		$ilCtrl->setParameterByClass('ilObjPluginDispatchGUI', 'ref_id', $this->getRefId());
		$url = $ilCtrl->getLinkTargetByClass(array('ilObjPluginDispatchGUI', 'ilObjQuestionSetPoolGUI'), '', '', false, false);
		foreach((array)$params as $key => $value)
		{
			$url = ilUtil::appendUrlParameterString($url, $key . '='. $value);
		}

		return $url;
	}

	/**
	 * 
	 */
	protected function initType()
	{
		require_once 'class.ilQuestionSetPoolPlugin.php';
		$this->setType(ilQuestionSetPoolPlugin::getPluginId());
	}

	protected function doRead()
	{
		global $DIC;
		$ilDB = $DIC->database();

		$this->set_list = new ilQuestionSetPoolSetList($this);

		$result = $ilDB->queryF(
			"SELECT * FROM rep_robj_xqsp_data WHERE id = %s",
			array("integer"),
			array($this->id)
		);

		$data = $ilDB->fetchAssoc($result);

		$this->setOnline($data["is_online"]);
		$this->setSkillServiceEnabled($data['skill_service']);
	}

	/**
	 * 
	 */
	public function doCreate()
	{
		global $DIC;
		$ilDB = $DIC->database();

		$this->set_list      = new ilQuestionSetPoolSetList($this);

		$ilDB->insert(
			$this->plugin->getAdditionalDataTable(),
			array(
				"id" => array("integer", $this->id),
				"is_online" => array("integer", $this->isOnline())
			)
		);

		$this->createMetaData();
	}

	/**
	 *
	 */
	public function doUpdate()
	{
		global $DIC;
		$ilDB = $DIC->database();
		
		$ilDB->update(
			$this->plugin->getAdditionalDataTable(),
			array(
				"is_online" => array("integer", $this->isOnline()),
				'skill_service' => array('integer', $this->isSkillServiceEnabled())
			),
			array(
				"id" => array("integer", $this->id)
			)
		);
		
		$this->updateMetaData();
	}

	/**
	 * Cascade delete children
	 */
	protected function beforeDelete()
	{
		$sets = $this->set_list->getData();
		foreach($sets as $set)
		{
			/**
			 * @var $set ilQuestionSetPoolSet
			 */
			$set->remove();
		}

		return true;
	}

	protected function doDelete()
	{
		global $DIC;
		$ilDB = $DIC->database();

		$query = 'DELETE FROM '.$this->plugin->getAdditionalDataTable().' WHERE id = ' . $ilDB->quote($this->getId(), 'integer');
		$ilDB->manipulate($query);

		$this->deleteMetaData();
	}

	

	/**
	 * @param ilObjQuestionSetPool  $new_obj
	 * @param int                   $a_target_id
	 * @param int                   $a_copy_id
	 */
	public function doCloneObject($new_obj, $a_target_id, $a_copy_id = NULL)
	{
		$this->cloneMetaData($new_obj);
		$this->cloneQuestionSets($new_obj);
	}
	
	/**
	 * @param ilObjQuestionSetPool| ilObjFormATest $new_obj
	 */
	public function cloneQuestionSets($new_obj)
	{
		$this->plugin->includeClass('models/class.ilQuestionSetPoolSet.php');
		$this->plugin->includeClass('models/class.ilQuestionSetPoolPath.php');
		$this->plugin->includeClass('models/class.ilQuestionSetPoolNode.php');
		$this->plugin->includeClass('models/class.ilQuestionSetPoolJumpCondition.php');
		
		$setlist = $this->getSetList();
		$sets    = $setlist->loadData();
		foreach((array)$sets as $set_index => $set_data)
		{
			$set = new ilQuestionSetPoolSet((int)$set_data['id']);
			$set->read();

			$new_set = new ilQuestionSetPoolSet();
			$new_set->setContainerObjId($new_obj->getId());
			$new_set->setTitle((string)$set->getTitle());
			$new_set->setDescription((string)$set->getDescription());
			$new_set->update();

			// Export QuestionList 
			$question_mapping = $this->cloneQuestionList($set, $new_set);

			// Export Paths
			$this->clonePathList($set->getPathList(), $new_set, $question_mapping);
		}
	}
	
	/**
	 * @param ilQuestionSetPoolSet $source_set
	 * @param ilQuestionSetPoolSet $new_set
	 * @return array
	 */
	protected function cloneQuestionList(ilQuestionSetPoolSet $source_set, ilQuestionSetPoolSet $new_set)
	{
		$questionList = $source_set->getQuestionList();
		$questions    = $questionList->loadData();

		$mapping = array();

		foreach((array)$questions as $question_index => $question)
		{
			$mapping = $questionList->cloneQuestionList(array($question['question_id']), $source_set, $new_set) + $mapping;
		}

		return $mapping;
	}
	
	/**
	 * @param ilQuestionSetPoolPathList $pathList
	 * @param ilQuestionSetPoolSet      $new_set
	 * @param array                     $question_mapping A map of integer values, old_question_fi -> new_question_fi
	 */
	protected function clonePathList(ilQuestionSetPoolPathList $pathList, ilQuestionSetPoolSet $new_set, array $question_mapping)
	{
		$paths = $pathList->loadData();
		foreach((array)$paths as $path_index => $path_data)
		{
			$path = new ilQuestionSetPoolPath();
			$path->setId($path_data['id']);
			$path->read();
			
			$new_path = new ilQuestionSetPoolPath();
			$new_path->setSet($new_set);
			$new_path->setTitle($path->getTitle());
			$new_path->setIndex($path->getIndex());
			$new_path->import();

			// Nodes 
			$this->cloneNodeList($path->getNodes(), $new_path, $question_mapping);
		}
	}
	
	/**
	 * @param ilQuestionSetPoolNodeList $nodeList
	 * @param ilQuestionSetPoolPath     $new_path
	 * @param array                     $question_mapping A map of integer values, old_question_fi -> new_question_fi
	 * $question_mapping A map of integer values, old_question_fi -> new_question_fi
	 */
	protected function cloneNodeList(ilQuestionSetPoolNodeList $nodeList, ilQuestionSetPoolPath $new_path, array $question_mapping)
	{
		$nodes = $nodeList->loadData();
		foreach((array)$nodes as $node_index => $node_data)
		{
			$node = new ilQuestionSetPoolNode((int)$node_data['id']);
			$node->read();

			$new_node = new ilQuestionSetPoolNode();
			if(isset($question_mapping[$node->getQuestion()->getId()]))
			{
				$new_node->setQuestion(assQuestion::_instantiateQuestion($question_mapping[$node->getQuestion()->getId()]));
			}
			else
			{
				$new_node->setQuestion(assQuestion::_instantiateQuestion($node->getQuestion()->getId()));
			}
			$new_node->setPath($new_path);
			$new_node->setInitialCondition($node->getInitialCondition());
			$new_node->setDominantScoring($node->getDominantScoring());
			$new_node->setTrueFeedback($node->getTrueFeedback());
			$new_node->setFalseFeedback($node->getFalseFeedback());
			$new_node->import();

			// Jump Conditions 
			$this->cloneJumpConditions($node->getJumpConditionList(), $new_node);
		}
	}
	
	/**
	 * @param ilQuestionSetPoolJumpConditionList $jumpConditionList
	 * @param ilQuestionSetPoolNode              $new_node
	 */
	protected function cloneJumpConditions(ilQuestionSetPoolJumpConditionList $jumpConditionList, ilQuestionSetPoolNode $new_node)
	{
		$jumpConditions = $jumpConditionList->getData();
		foreach((array)$jumpConditions as $jmp_con_index => $jumpCondition_data)
		{
			$jumpCondition = new ilQuestionSetPoolJumpCondition();
			$jumpCondition->setId($jumpCondition_data['id']);
			$jumpCondition->read();
			
			$new_jump_con = new ilQuestionSetPoolJumpCondition();
			$new_jump_con->setNode($new_node);
			$new_jump_con->setQuestion($new_node->getQuestion());
			$new_jump_con->setJumpCondition($jumpCondition->getJumpCondition());
			$new_jump_con->setRepititions($jumpCondition->getRepititions());
			$new_jump_con->setTrueFeedback($jumpCondition->getTrueFeedback());
			$new_jump_con->setTrueJumpFeedback($jumpCondition->getTrueJumpFeedback());
			$new_jump_con->setFalseFeedback($jumpCondition->getFalseFeedback());

			$new_jump_con->import();
		}
	}
	
	/**
	 * @param boolean $online
	 */
	public function setOnline($online)
	{
		$this->online = $online;
	}

	/**
	 * @return boolean
	 */
	public function isOnline()
	{
		return $this->online;
	}

	/**
	 * @return ilQuestionSetPoolSetList
	 */
	public function getSetList()
	{
		return $this->set_list;
	}

	/**
	 * @return array
	 */
	public function toArray()
	{
		return array(
			'title'         => $this->getTitle(),
			'description'   => $this->getDescription(),
			'online'        => $this->isOnline(),
			'skill_service' => $this->isSkillServiceEnabled() 
		);
	}

	/**
	 * @param ilPropertyFormGUI $form
	 */
	public function bindForm(ilPropertyFormGUI $form)
	{
		$this->setTitle($form->getInput('title'));
		$this->setDescription($form->getInput('description'));
		$this->setOnline($form->getInput('online'));
		$this->setSkillServiceEnabled((bool)$form->getInput('skill_service'));
	}

	/**
	 * @param int $obj_id
	 * @return bool
	 */
	public static function getOnlineStatusByObjId($obj_id)
	{
		if(isset(self::$online_status_map[$obj_id]))
		{
			return (bool)self::$online_status_map[$obj_id];
		}

		self::preloadDataForLists(array($obj_id));
	}

	/**
	 * @param array $obj_ids
	 * @param array $ref_ids
	 */
	public static function preloadDataForLists(array $obj_ids, array $ref_ids = array())
	{
		global $DIC;
		$ilDB = $DIC->database();

		if(is_array($obj_ids) && count($obj_ids) == 1)
		{
			$in = ' id = ' . $ilDB->quote(current($obj_ids), 'integer') . ' ';
		}
		else
		{
			$in = $ilDB->in('id', $obj_ids, false, 'integer');
		}
		
		$query = "
			SELECT id, is_online
			FROM rep_robj_xqsp_data
			WHERE $in 
		";
		$res   = $ilDB->query($query);

		while($row = $ilDB->fetchAssoc($res))
		{
			self::$online_status_map[$row['id']] = $row['is_online'];
		}
	}

	public static function getAvailableQuestionSetPools($could_be_offline = FALSE)
	{
		global $DIC;
		$ilDB = $DIC->database();
		$ilUser = $DIC->user();
		
		$result_array = array();

		$pools = ilUtil::_getObjectsByOperations("xqsp", "read", $ilUser->getId(), -1);
		$obj_ids = array();
		$ref_ids = array();
		foreach($pools as $ref_id)
		{
			$obj_ids[$ref_id] = ilObject::_lookupObjId($ref_id);
			$ref_ids[ilObject::_lookupObjId($ref_id)] = $ref_id;
		}

		if($could_be_offline)
		{
			$result = $ilDB->query("SELECT id FROM rep_robj_xqsp_data WHERE " . $ilDB->in("id", $obj_ids, false, "integer"));		
		}
		else
		{
			$result = $ilDB->queryF(
				"SELECT id FROM rep_robj_xqsp_data WHERE is_online = %s AND " . $ilDB->in("id", $obj_ids, false, "integer"),
				array("integer"),
				array(1)
			);
		}
		
		while(($row = $ilDB->fetchAssoc($result)) != null)
		{
			$pool = new ilObjQuestionSetPool();
			$pool->setRefId($ref_ids[$row["id"]]);
			$pool->read();
			$result_array[$row["id"]] = $pool;
		}

		return $result_array;
	}

	public static function canSynchronizeSet($set_id)
	{
		global $DIC; 
		$rbacsystem = $DIC->rbac()->system();

		if($set_id == null)
		{
			return false;
		}

		$set = new ilQuestionSetPoolSet($set_id);
		$set->read();
		if($set->getContainerObjId() != null)
		{
			$available = array_keys(self::getAvailableQuestionSetPools());

			if(in_array($set->getContainerObjId(), $available))
			{
				$refs = ilObject::_getAllReferences($set->getContainerObjId());
				foreach($refs as $ref)
				{
					if($rbacsystem->checkAccess("write", $ref) && ilObject::_hasUntrashedReference($set->getContainerObjId()))
					{
						return true;
					}
				}
			}
		}

		return false;
	}

	/**
	 * @param boolean $skillServiceEnabled
	 */
	public function setSkillServiceEnabled($skillServiceEnabled)
	{
		$this->skillServiceEnabled = $skillServiceEnabled;
	}

	/**
	 * @return bool
	 */
	public function isSkillServiceEnabled()
	{
		if(!self::isSkillManagementGloballyActivated())
		{
			return false;
		}
		
		return $this->skillServiceEnabled;
	}

	/**
	 * @return bool
	 */
	public static function isSkillManagementGloballyActivated()
	{
		
		if( self::$isSkillManagementGloballyActivated === null )
		{
			include_once './Services/Skill/classes/class.ilSkillManagementSettings.php';
			$skmgSet = new ilSkillManagementSettings();

			self::$isSkillManagementGloballyActivated = $skmgSet->isActivated();
		}

		return self::$isSkillManagementGloballyActivated;
	}
}
